#!/usr/bin/perl -w

package RPM::Source::Convert::Plugins::DistroMap::SourceAnalyzer;
use strict;
use warnings;
use Carp;

use DistroMap;
use File::Basename;
use RPM::Source::Tools::SourceBundle;
use RPM::Source::Tools::SourceAnalyzer;

use RPM::Source::Transform;
use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::DistroMapResource;
use RPM::Source::Transformation::BuildreqFactory;
our @ISA=qw/RPM::Source::Transformation::Component/;

push @RPM::Source::Transform::FACTORY_CLASSES, __PACKAGE__;

$DistroMap::opt_dest_check_distrodb=7; # days

# TODO!
push @RPM::Source::Transform::EXT_OPTS, @RPM::Source::Tools::SourceAnalyzer::LONGOPT;
push @RPM::Source::Transform::EXT_OPTS_USAGE, \&RPM::Source::Tools::SourceAnalyzer::longopt_usage;

our $verbose=0;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'distromap_source_analyzer',
	    -opt_handler_sub=>\&opt_handler_distromap_source_analyzer),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_DEFAULT,
	    -name => 'distromap_source_analyzer_resolve_umbiguous_pkgconfig',
	    -opt_handler_sub=>\&opt_handler_distromap_source_analyzer_resolve_umbiguous_pkgconfig)
	;
}

sub opt_handler_distromap_source_analyzer {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();

    unless (defined $spec->{SOURCEDIR}) {
	print STDERR "WARNING: SourceAnalyzer: sourcedir is not defined. Can't find the sources.\n";
	return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
    }
    my $name=$spec->main_section->get_tag('Name');
    if ($name =~ /^mingw/) {
	print STDERR "info: SourceAnalyzer: mingw support is not implemented.\n" if $RPM::Source::Transform::verbose;
	return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
    }

    my ($existing,$existing_pre)=$resourceobj->buildreq->current_buildreq($spec);
    my $analyzer=RPM::Source::Tools::SourceAnalyzer->new(
	-verbose => $RPM::Source::Transform::verbose,
	-hint => 'convert',
	-distroinfo=> $resourceobj->get_distromap->dest(),
	-buildreq_storage => $resourceobj->buildreq,
	-current_buildreq => $existing,
	-scan_configure_for_broken_cpp_check => 1,
	-select_multiple_dependency => {
	    '/usr/bin/flex' => 'flex',
	    '/usr/bin/flex++' => 'flex',
	    'net' => 'libnet2-devel',
	    'png' => 'libpng-devel',
	    'png.h' => 'libpng-devel',
	    'raw1394' => 'libraw1394-devel',
	    'libraw1394/raw1394.h' => 'libraw1394-devel',
	    'xercesc/util/XercesDefs.hpp' => 'libxerces-c-devel',
	    'cpufreq.h' => 'libcpufreq-devel',
	    'db-4.7' => 'libdb4.8-devel',
	    'db-4.8' => 'libdb4.8-devel',
	    'db.h' => 'libdb4.8-devel',
	}
	);
    foreach my $srcitem (@{$spec->raw_sourcelist_ref}) {
	next unless $srcitem;
	$srcitem=$macros->macro_subst($srcitem);
	# due to something like %{download_root}MTLc3m.ttf
	my $filename=$spec->{SOURCEDIR}.'/'.basename($srcitem);
	print STDERR "DistroMap::SourceAnalyzer: no file $filename:\n" if not -e $filename and $RPM::Source::Transform::verbose;
	return RPM::Source::Transformation::Component::TRANSFORMATION_OK unless -e $filename;
	my $tarball=RPM::Source::Tools::SourceBundle->create($filename);
	$analyzer->analyze($tarball);
    }
    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}

# TODO! Disrtomap it!
my %dep_subst_hack=qw/
pkgconfig(pygobject-2.0) python-module-pygobject-devel
perl(mro.pm) perl-base
/;

my %dep_drop_hack=map{$_=>1} qw/
pkgconfig(hal)
/;

sub opt_handler_distromap_source_analyzer_resolve_umbiguous_pkgconfig {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $buildreq = $resourceobj->buildreq;
    #my $distrodb = $resourceobj->distromap->dest;
    foreach my $dep ($buildreq->get_list) {
	if ($dep_subst_hack{$dep}) {
	    $buildreq->del($dep);
	    $buildreq->add_for("replaced automatic dependency $dep", $dep_subst_hack{$dep});
	    print STDERR "info: resolved dependency $dep into $dep_subst_hack{$dep}.\n" if $RPM::Source::Transform::verbose;
	} elsif ($dep_drop_hack{$dep}) {
	    $buildreq->del($dep);
	    print STDERR "info: dropped dependency $dep\n" if $RPM::Source::Transform::verbose;
	}
    }
    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}


1;
