#!/usr/bin/perl -w

package RPM::Source::Convert::Plugins::DistroMap::SubpackageRename;
use strict;
use warnings;
use Carp;

use RPM::Source::Transform;
use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::DistroMapResource;
our @ISA=qw/RPM::Source::Transformation::Component/;

push @RPM::Source::Transform::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'distromap_subpackage_rename',
	    -opt_handler_sub=>\&opt_handler_distromap_subpackage_rename);
}

sub opt_handler_distromap_subpackage_rename {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    my @sections=$spec->get_sections();
    my (%rename, %keepname);
    foreach my $sec (@sections) {
	my $sec_pkg = $sec->get_raw_package();
	# subsection rename. skip main section
	next unless $sec_pkg;
	my $sec_pkg_expanded = $sec->get_package_name();
	next if $keepname{$sec_pkg_expanded};
	if (not $rename{$sec_pkg_expanded}) {
	    if ($sec_pkg_expanded =~/-fonts(?:-common|-ghostscript)?$/ or
		$sec_pkg_expanded =~/^(?:tex|python[23]?)-/ or
		$sec_pkg_expanded =~/-(?:python[23]?|pygobject2|firmware|libs|theme)$/
		) {
		print STDERR "rename candidate detected: $sec_pkg ($sec_pkg_expanded)\n" if $verbose;
		my @mapname=$distromap->multimap_binary_origin2dest($sec_pkg_expanded);
		if ($mapname[0]) {
		    # if no protection from renaming
		    if ($mapname[0] ne $sec_pkg_expanded) {
			print STDERR "distromap: renaming $sec_pkg_expanded -> @mapname\n" if $RPM::Source::Transform::verbose>1;
			$rename{$sec_pkg_expanded}=$mapname[0];
		    }
		} elsif ($sec_pkg_expanded !~/^python[23]?-module-/ and (
		    $sec_pkg_expanded =~/^python[23]?-/ or
		    $sec_pkg_expanded =~/-python[23]?$/
		    )) {
		    my $newname=$sec_pkg_expanded;
		    $newname=~s/-?python([23]?)-?//;
		    $newname='python'.$1.'-module-'.$newname;
		    $rename{$sec_pkg_expanded}=$newname;
		    print STDERR "python policy: renaming $sec_pkg_expanded -> $newname\n" if $RPM::Source::Transform::verbose>1;
		} else {
		    $keepname{$sec_pkg_expanded}=1;
		}
	    } else {
		$keepname{$sec_pkg_expanded}=1;
	    }
	}
	if ($rename{$sec_pkg_expanded}) {
	    $sec->set_raw_package('-n '.$rename{$sec_pkg_expanded});
	    my $sec_pkg = $sec->get_raw_package();
	    print STDERR "distromap: ".$sec->get_type." renaming $sec_pkg_expanded -> $sec_pkg\n" if $RPM::Source::Transform::verbose>2;
	}
    }
    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}

1;
