#!/usr/bin/perl -w

package RPM::Source::Convert::Plugins::Mageia2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transform;
use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::DistroMapResource;
use RPM::Source::Convert::Plugins::Generic2ALT;
use RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;


our @ISA=qw/RPM::Source::Transformation::Component/;
push @RPM::Source::Transform::FACTORY_CLASSES, __PACKAGE__;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'mageia2alt_convert',
	    -opt_handler_sub=>\&opt_handler_mageia2alt_convert);
}

sub opt_handler_mageia2alt_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;

    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;

    my $has_perl_convert_version=0;
    my @__push_mainsec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    $sec->map_body(sub {
		s/\%mkrel\s+//g;
		# mandrake mklibname macros
		#s/^(\s*\%define\s+\S+\s+)\%mklibname\s+(\S+)(?:\s+(\S+))?(.*)$/$1&__mk/
		s/\%mklibname\s+(\S+)(?:\s+(\S+))?/&__mklibname($1,$2)/ge;

		# our perl policy is different
		$has_perl_convert_version=1 if s/\%perl_convert_version\s+//g;

#		if (/^BuildRequires:/i) {
#		} elsif (/^Requires:/i) {
#		}
			   } );
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    $sec->map_body(sub {
		s,\%make_install(?=(?:\s|$)),%make_install DESTDIR=%buildroot install,g
			   });
	} elsif ($t eq 'triggerun') {
	    if ($sec->match_body(qr'(?:chkconfig|systemctl)')) {
		$sec->delete;
		next;
	    }
	} elsif ($t eq 'post') {
	} elsif ($t eq 'files') {
	    #
	}

	$sec->map_body(sub {
	    s,\%configure2_5x,%configure,g;
	} ) if ($t eq 'install' or $t eq 'build' or $t eq 'prep');
    }
    $mainsec->push_body(@__push_mainsec) if @__push_mainsec;
    if ($has_perl_convert_version) {
	my $macros=$spec->macros();
	my $ver=$macros->get_raw('version');
	$ver=~s/^(?:\%perl_convert_version)?\s+//;
	$macros->set_raw('version',$ver);
    }

    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}

sub __mklibname {
    my ($arg1,$arg2)=@_;
    $arg1=&__brace_macro($arg1);
    $arg2=&__brace_macro($arg2);
    my $out;
    if ($arg1 eq '-d') {
	$out=$arg2.'-devel';
    } elsif (defined $arg2) {
	$out='lib'.$arg1.$arg2;
    } else {
	$out='lib'.$arg1;
    }
    return $out;
}


sub __brace_macro {
    my $in=shift;
    if (defined $in and $in =~s/^\%([^{])/$1/) {
	$in='%{'.$in.'}';
    }
    return $in;
}

1;
__END__
