#!/usr/bin/perl -w

package RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transform;
use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::OptHandlerWrapper;

our @ISA=qw/RPM::Source::Transformation::Component/;
push @RPM::Source::Transform::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'fedora2alt_convert',
	    -opt_handler_sub=>\&opt_handler_perlstyle_use2path_convert);
}

sub __convert2alt_fedora_perl_provides {
    my $prov=shift;
    #my $old=$prov;
    $prov=~s!::!/!g if $prov=~/perl\(/;
    $prov=~s!perl\(([^\)]+)\)!perl($1.pm)!g;
    #print STDERR "CONVERTED: $old -> $prov\n";
    return $prov;
}

sub __convert2alt_fedora_perl_version {
    my $fcperlver=shift;
    $fcperlver=~s/^(\d+\.\d\d\d)(\d\d\d)/$1.${2}/g;
    $fcperlver=~s/^(\d+\.\d\d\d)(\d\d)/$1.${2}0/g;
    $fcperlver=~s/^(\d+\.\d\d\d)(\d)/$1.${2}00/g;
    $fcperlver=~s/\.000\.0+$/.0/;
    $fcperlver=~s/\.000$/.0/;
    return $fcperlver;
}

sub opt_handler_perlstyle_use2path_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;

    my ($flag_macro_perl,$flag_catalyst_pl);
    my ($filter_from_provreq_is_used,$perl_buildsystem_is_used);
    my @__push_mainsec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    # fedora perl packaging
	    #Requires:       perl(:MODULE_COMPAT_%(eval "`%{__perl} -V:version`"; echo $version))
#############	    # $sec->map_body(sub { s/^Requires:\s*$// if s/perl\(:MODULE_COMPAT_\%\(eval[^\)]+\)\)//} );

	    $sec->map_body(sub {
		    if (/^BuildRequires:/i) {
			# TMP HACK TO DELETE!!! til https://bugzilla.altlinux.org/show_bug.cgi?id=27367
			s/(perl\([^\)]+\))\s+[>=]+?\s+v?[\d\.]+/$1/g;
		    } elsif (/^\%req_both\s/) { # %req_both expands to BR:+R:
			s/(perl\([^\)]+\))(?:\s+[>=]+?\s+v?[\d\.]+)?/&__convert2alt_fedora_perl_provides($1)/ge;
		    } elsif (/^Requires:/i) {
			# our ddd.ddd perl provides
			s/(perl\([^\)]+\)\s+[>=]+\s+)([\d\.]+)/$1.&__convert2alt_fedora_perl_version($2)/ge;
		    }
			   } );

	} elsif ($t eq 'prep') {
	    $sec->map_body(sub {
		$flag_catalyst_pl=1 if /catalyst.pl(?:\s|$)/;
			   });

	} elsif ($t eq 'build') {
	    $sec->map_body(sub {
		if (/^(.*)Makefile.PL\s+/) {
		    my $precmd=$1;
		    # %{__perl} Makefile.PL INSTALLDIRS=vendor|perl
		    if ($precmd=~/perl/) {
			s/Makefile.PL\s+/Makefile.PL INSTALLMAN1DIR=%_man1dir /;
			$perl_buildsystem_is_used=1;
		    }
		} elsif (/^(.*)Build.PL\s+/) {
		    my $precmd=$1;
		    if ($precmd=~/perl/) {
			# %{__perl} Build.PL installdirs=vendor --install_path bindoc=%_man1dir
			s/Build.PL\s+/Build.PL --install_path bindoc=%_man1dir /;
			$perl_buildsystem_is_used=1;
		    }
		}
			   } );


	} elsif ($t eq 'files') {
	    #
	}

	# general hacks here
	$sec->map_body(sub {
	    if (/\%{?perl_(?:arch|priv|vendor)lib}?/) {
		s,%{perl_archlib},%{perl_vendor_archlib},g;
		s,%perl_archlib,%perl_vendor_archlib,g;
		s,%{perl_vendorarch},%{perl_vendor_archlib},g;
		s,%perl_vendorarch,%perl_vendor_archlib,g;
		s,%{perl_privlib},%{perl_vendor_privlib},g;
		s,%perl_privlib,%perl_vendor_privlib,g;
		s,%{perl_vendorlib},%{perl_vendor_privlib},g;
		s,%perl_vendorlib,%perl_vendor_privlib,g;
		$flag_macro_perl=1;
	    }
	} );
    }

    if ($perl_buildsystem_is_used) {
	foreach my $sec ($spec->get_sections()) {
	    my $type = $sec->get_type();
	    if ($type eq 'files') {
		# %exclude %{_mandir}/man3/DateTime.*
		$sec->exclude(qr'^\s*(\%exclude\s+)?\%{_mandir}/man3/');
		#%{_mandir}/man?/Net::Server::Coro*.*
		$sec->exclude(qr'^\s*(\%exclude\s+)?\%{_mandir}/man\?/\w+::');
	    }
	}
    }


    $mainsec->push_body(@__push_mainsec) if @__push_mainsec;
    my $buildreq=$resourceobj->buildreq;
    $buildreq->pre_add('rpm-build-perl') if $flag_macro_perl;
    $buildreq->add_for('used in spec file','/usr/bin/catalyst.pl') if $flag_catalyst_pl;
    # TODO: implement if $resourceobj -> lang -> perl
    $buildreq->add('perl-podlators') if $flag_macro_perl or $buildreq->has('perl-devel');

    # crux fixed it in perl-Test-Most
#    map {$buildreq->add($_)} qw!
#perl(Test/Exception.pm)
#perl(Test/Differences.pm)
#perl(Test/Deep.pm)
#perl(Test/Warn.pm)
#perl(Test/More.pm)
#! if $buildreq->has('perl(Test/Most.pm)');

    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}

1;
__END__
